# Test the SQLite writer object.


import re
import unittest
import datetime
import StringIO

from netsa.data.times import make_datetime

from fake import FakeAlert, FakeWriterOperations

from pipevis import SqlWriter


def normalize_sql(s):
    # Reduce all whitespace in s to single spaces
    return re.sub(r"""\s+""", " ", s).lower()
    

        


reference_alert = {
    "generator" : "org.cert.netsa.pipeline",
    "generator_version" : "0.1.2",
    "timestamp" : make_datetime("2011/01/01:02:14:00"),
    "severity": 1,
    "tags" : ["type:watchlist", "name:test-watch"],
    "data" : "{extra_info: 'A JSON blob of stuff'}"
}

class WriterOperationsObserver(object):
    def __init__(self):
        self._alert_inserted = None
        self._bin_inserted = None
        self._bin_updated = None
        self._commited = None
        def on_insert_alert(alert, alert_type, alert_name):
            self._alert_inserted = (alert, alert_type, alert_name)
        def on_insert_bin(bin_timestamp, alert_type, alert_name):
            self._bin_inserted = (bin_timestamp, alert_type, alert_name)
        def on_update_bin(bin_timestamp, alert_type, alert_name):
            self._bin_updated = (bin_timestamp, alert_type, alert_name)
        def on_commit():
            self._commited = True
        self._ops = FakeWriterOperations(insert_alert=on_insert_alert,
                                         insert_bin=on_insert_bin,
                                         update_bin=on_update_bin,
                                         commit=on_commit)
    def get_writer_ops(self): return self._ops
    def test_get_alert_inserted(self): return self._alert_inserted
    def test_get_bin_inserted(self): return self._bin_inserted
    def test_get_bin_updated(self): return self._bin_updated
    

class TestWriter(unittest.TestCase):
    def test_basic(self):
        alert = FakeAlert(**reference_alert)
        observer = WriterOperationsObserver()
        writer = SqlWriter(writeroperations=observer.get_writer_ops(),
                           bin_size=datetime.timedelta(seconds=300))

        writer(alert)

        bin_timestamp =  make_datetime("2011/01/01:02:10:00")
        exp_type = "watchlist"
        exp_name = "test-watch"

        inserted_alert, a_type, a_name = observer.test_get_alert_inserted()

        self.assertEqual(id(inserted_alert), id(alert))
        self.assertEqual((a_type, a_name), (exp_type, exp_name))
        inserted_bin, a_type, a_name = observer.test_get_bin_inserted()
        self.assertEqual(inserted_bin, bin_timestamp)
        self.assertEqual((a_type, a_name), (exp_type, exp_name))

        updated_bin, a_type, a_name = observer.test_get_bin_updated()
        self.assertEqual(updated_bin, bin_timestamp)
        self.assertEqual((a_type, a_name), (exp_type, exp_name))
        


if __name__ == '__main__':
    unittest.main()
